/****************************************************************************
** Meta object code from reading C++ file 'programmodule.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/widgets/programmodule.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'programmodule.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ProgramModuleE_t {};
} // unnamed namespace

template <> constexpr inline auto ProgramModule::qt_create_metaobjectdata<qt_meta_tag_ZN13ProgramModuleE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ProgramModule",
        "activateWindow",
        "",
        "handleKeypadButtonPress",
        "key",
        "handleKeypadButtonPressByspace",
        "shiftArrowListWidgetItemClicked",
        "row",
        "isselect",
        "shiftArrowListWidgetItemSpace",
        "shiftArrowListWidgetItemPressed",
        "byteArrowListWidgetItemClicked",
        "byteArrowListWidgetItemSpace",
        "byteArrowListWidgetItemPressed",
        "checkBtnKeypadThemeChange",
        "type",
        "radixListChange",
        "QModelIndex",
        "index",
        "isspace",
        "mAvailableEvent",
        "mUnAvailableEvent",
        "showListWidget",
        "closeListWidget"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activateWindow'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleKeypadButtonPress'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'handleKeypadButtonPressByspace'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'shiftArrowListWidgetItemClicked'
        QtMocHelpers::SlotData<void(int, bool)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'shiftArrowListWidgetItemSpace'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'shiftArrowListWidgetItemPressed'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'byteArrowListWidgetItemClicked'
        QtMocHelpers::SlotData<void(int, bool)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Bool, 8 },
        }}),
        // Slot 'byteArrowListWidgetItemSpace'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'byteArrowListWidgetItemPressed'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'checkBtnKeypadThemeChange'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'radixListChange'
        QtMocHelpers::SlotData<void(const QModelIndex &, bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'mAvailableEvent'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'mUnAvailableEvent'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showListWidget'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeListWidget'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProgramModule, qt_meta_tag_ZN13ProgramModuleE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ProgramModule::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProgramModuleE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProgramModuleE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ProgramModuleE_t>.metaTypes,
    nullptr
} };

void ProgramModule::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProgramModule *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activateWindow(); break;
        case 1: _t->handleKeypadButtonPress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->handleKeypadButtonPressByspace((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->shiftArrowListWidgetItemClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->shiftArrowListWidgetItemSpace(); break;
        case 5: _t->shiftArrowListWidgetItemPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->byteArrowListWidgetItemClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 7: _t->byteArrowListWidgetItemSpace(); break;
        case 8: _t->byteArrowListWidgetItemPressed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->checkBtnKeypadThemeChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->radixListChange((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 11: _t->mAvailableEvent(); break;
        case 12: _t->mUnAvailableEvent(); break;
        case 13: _t->showListWidget(); break;
        case 14: _t->closeListWidget(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProgramModule::*)()>(_a, &ProgramModule::activateWindow, 0))
            return;
    }
}

const QMetaObject *ProgramModule::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProgramModule::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ProgramModuleE_t>.strings))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int ProgramModule::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void ProgramModule::activateWindow()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
