/****************************************************************************
** Meta object code from reading C++ file 'inputedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/widgets/inputedit.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9InputEditE_t {};
} // unnamed namespace

template <> constexpr inline auto InputEdit::qt_create_metaobjectdata<qt_meta_tag_ZN9InputEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InputEdit",
        "keyPress",
        "",
        "QKeyEvent*",
        "equal",
        "undo",
        "redo",
        "paste",
        "copy",
        "cut",
        "selectAllText",
        "deleteText",
        "setResult",
        "emptyExpression",
        "b",
        "cursorPositionNumberChanged",
        "num",
        "prolistAns",
        "std::pair<bool,Quantity>",
        "pair",
        "swietThreeSeparate",
        "swietFourSeparate",
        "separateChange",
        "setUndoAction",
        "state",
        "setRedoAction",
        "symbolFaultTolerance",
        "text",
        "isExpressionEmpty",
        "hisexpression",
        "autoZoomFontSize",
        "themetypechanged",
        "type",
        "valueChangeFromProSyskeypad",
        "handleTextChanged",
        "radixChanged",
        "baseori",
        "basedest",
        "scanAndExec",
        "onSwietThreeSeparateClicked",
        "onswietFourSeparateClicked",
        "initAction",
        "handleCursorPositionChanged",
        "oldPos",
        "newPos",
        "BracketCompletion",
        "e",
        "pointFaultTolerance",
        "isSymbol",
        "multipleArithmetic",
        "QString&",
        "showTextEditMenu",
        "selectionChangedSlot",
        "showTextEditMenuByAltM",
        "formatAns"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyPress'
        QtMocHelpers::SignalData<void(QKeyEvent *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'equal'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'undo'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'redo'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'paste'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'copy'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cut'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectAllText'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deleteText'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'setResult'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'emptyExpression'
        QtMocHelpers::SignalData<void(bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'cursorPositionNumberChanged'
        QtMocHelpers::SignalData<void(QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Signal 'prolistAns'
        QtMocHelpers::SignalData<void(QPair<bool,Quantity>)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'swietThreeSeparate'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'swietFourSeparate'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'separateChange'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setUndoAction'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Slot 'setRedoAction'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Slot 'symbolFaultTolerance'
        QtMocHelpers::SlotData<QString(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'isExpressionEmpty'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hisexpression'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'autoZoomFontSize'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'themetypechanged'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
        // Slot 'valueChangeFromProSyskeypad'
        QtMocHelpers::SlotData<void(const QString)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'handleTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'radixChanged'
        QtMocHelpers::SlotData<void(int, int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 36 }, { QMetaType::Int, 37 },
        }}),
        // Slot 'scanAndExec'
        QtMocHelpers::SlotData<QString(int, int)>(38, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 36 }, { QMetaType::Int, 37 },
        }}),
        // Slot 'onSwietThreeSeparateClicked'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onswietFourSeparateClicked'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initAction'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleCursorPositionChanged'
        QtMocHelpers::SlotData<void(int, int)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 43 }, { QMetaType::Int, 44 },
        }}),
        // Slot 'BracketCompletion'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 3, 46 },
        }}),
        // Slot 'pointFaultTolerance'
        QtMocHelpers::SlotData<QString(const QString &)>(47, 2, QMC::AccessPrivate, QMetaType::QString, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'isSymbol'
        QtMocHelpers::SlotData<bool(const QString &)>(48, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QString, 27 },
        }}),
        // Slot 'multipleArithmetic'
        QtMocHelpers::SlotData<void(QString &)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 50, 27 },
        }}),
        // Slot 'showTextEditMenu'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'selectionChangedSlot'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showTextEditMenuByAltM'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'formatAns'
        QtMocHelpers::SlotData<QString(const QString &)>(54, 2, QMC::AccessPrivate, QMetaType::QString, {{
            { QMetaType::QString, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<InputEdit, qt_meta_tag_ZN9InputEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject InputEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9InputEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9InputEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9InputEditE_t>.metaTypes,
    nullptr
} };

void InputEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InputEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyPress((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 1: _t->equal(); break;
        case 2: _t->undo(); break;
        case 3: _t->redo(); break;
        case 4: _t->paste(); break;
        case 5: _t->copy(); break;
        case 6: _t->cut(); break;
        case 7: _t->selectAllText(); break;
        case 8: _t->deleteText(); break;
        case 9: _t->setResult(); break;
        case 10: _t->emptyExpression((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->cursorPositionNumberChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->prolistAns((*reinterpret_cast<std::add_pointer_t<std::pair<bool,Quantity>>>(_a[1]))); break;
        case 13: _t->swietThreeSeparate(); break;
        case 14: _t->swietFourSeparate(); break;
        case 15: _t->separateChange(); break;
        case 16: _t->setUndoAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setRedoAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: { QString _r = _t->symbolFaultTolerance((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 19: _t->isExpressionEmpty(); break;
        case 20: _t->hisexpression(); break;
        case 21: _t->autoZoomFontSize(); break;
        case 22: _t->themetypechanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->valueChangeFromProSyskeypad((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->handleTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->radixChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 26: { QString _r = _t->scanAndExec((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->onSwietThreeSeparateClicked(); break;
        case 28: _t->onswietFourSeparateClicked(); break;
        case 29: _t->initAction(); break;
        case 30: _t->handleCursorPositionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 31: _t->BracketCompletion((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 32: { QString _r = _t->pointFaultTolerance((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->isSymbol((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: _t->multipleArithmetic((*reinterpret_cast<std::add_pointer_t<QString&>>(_a[1]))); break;
        case 35: _t->showTextEditMenu(); break;
        case 36: _t->selectionChangedSlot(); break;
        case 37: _t->showTextEditMenuByAltM(); break;
        case 38: { QString _r = _t->formatAns((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)(QKeyEvent * )>(_a, &InputEdit::keyPress, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::equal, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::undo, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::redo, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::paste, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::copy, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::cut, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::selectAllText, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::deleteText, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::setResult, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)(bool )>(_a, &InputEdit::emptyExpression, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)(QString )>(_a, &InputEdit::cursorPositionNumberChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)(QPair<bool,Quantity> )>(_a, &InputEdit::prolistAns, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::swietThreeSeparate, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::swietFourSeparate, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (InputEdit::*)()>(_a, &InputEdit::separateChange, 15))
            return;
    }
}

const QMetaObject *InputEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InputEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9InputEditE_t>.strings))
        return static_cast<void*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int InputEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 39;
    }
    return _id;
}

// SIGNAL 0
void InputEdit::keyPress(QKeyEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void InputEdit::equal()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void InputEdit::undo()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void InputEdit::redo()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void InputEdit::paste()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void InputEdit::copy()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void InputEdit::cut()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void InputEdit::selectAllText()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void InputEdit::deleteText()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void InputEdit::setResult()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void InputEdit::emptyExpression(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void InputEdit::cursorPositionNumberChanged(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void InputEdit::prolistAns(QPair<bool,Quantity> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void InputEdit::swietThreeSeparate()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void InputEdit::swietFourSeparate()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void InputEdit::separateChange()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
