/****************************************************************************
** Meta object code from reading C++ file 'memorywidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/views/memorywidget.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'memorywidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12MemoryWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto MemoryWidget::qt_create_metaobjectdata<qt_meta_tag_ZN12MemoryWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MemoryWidget",
        "widgetplus",
        "",
        "row",
        "widgetminus",
        "insidewidget",
        "itemclick",
        "std::pair<QString,Quantity>",
        "themechange",
        "type",
        "widgetclean",
        "mode",
        "ismenu",
        "hideWidget",
        "setThemeType",
        "expressionempty",
        "b",
        "widgetcleanslot",
        "setMemoryPublic",
        "MemoryPublic*",
        "pub",
        "resetItemHeight"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'widgetplus'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'widgetminus'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'insidewidget'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'itemclick'
        QtMocHelpers::SignalData<void(const QPair<QString,Quantity>)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 2 },
        }}),
        // Signal 'themechange'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Signal 'widgetclean'
        QtMocHelpers::SignalData<void(int, int, bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'hideWidget'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setThemeType'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'expressionempty'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'widgetcleanslot'
        QtMocHelpers::SlotData<void(int, int, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Slot 'setMemoryPublic'
        QtMocHelpers::SlotData<void(MemoryPublic *)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'resetItemHeight'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MemoryWidget, qt_meta_tag_ZN12MemoryWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MemoryWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MemoryWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MemoryWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12MemoryWidgetE_t>.metaTypes,
    nullptr
} };

void MemoryWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MemoryWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->widgetplus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->widgetminus((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->insidewidget(); break;
        case 3: _t->itemclick((*reinterpret_cast<std::add_pointer_t<std::pair<QString,Quantity>>>(_a[1]))); break;
        case 4: _t->themechange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->widgetclean((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 6: _t->hideWidget(); break;
        case 7: _t->setThemeType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->expressionempty((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->widgetcleanslot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 10: _t->setMemoryPublic((*reinterpret_cast<std::add_pointer_t<MemoryPublic*>>(_a[1]))); break;
        case 11: _t->resetItemHeight(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)(int )>(_a, &MemoryWidget::widgetplus, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)(int )>(_a, &MemoryWidget::widgetminus, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)()>(_a, &MemoryWidget::insidewidget, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)(const QPair<QString,Quantity> )>(_a, &MemoryWidget::itemclick, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)(int )>(_a, &MemoryWidget::themechange, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)(int , int , bool )>(_a, &MemoryWidget::widgetclean, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MemoryWidget::*)()>(_a, &MemoryWidget::hideWidget, 6))
            return;
    }
}

const QMetaObject *MemoryWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MemoryWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MemoryWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MemoryWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void MemoryWidget::widgetplus(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MemoryWidget::widgetminus(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MemoryWidget::insidewidget()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MemoryWidget::itemclick(const QPair<QString,Quantity> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MemoryWidget::themechange(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MemoryWidget::widgetclean(int _t1, int _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3);
}

// SIGNAL 6
void MemoryWidget::hideWidget()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
