/****************************************************************************
** Meta object code from reading C++ file 'sciexpressionbar.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/widgets/sciexpressionbar.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'sciexpressionbar.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16SciExpressionBarE_t {};
} // unnamed namespace

template <> constexpr inline auto SciExpressionBar::qt_create_metaobjectdata<qt_meta_tag_ZN16SciExpressionBarE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SciExpressionBar",
        "keyPress",
        "",
        "QKeyEvent*",
        "clearStateChanged",
        "themeChange",
        "type",
        "turnDeg",
        "fEStateChanged",
        "enterNumberEvent",
        "text",
        "enterSymbolEvent",
        "enterPointEvent",
        "enterBackspaceEvent",
        "enterClearEvent",
        "enterEqualEvent",
        "enterPercentEvent",
        "enterDegEvent",
        "mod",
        "enterFEEvent",
        "isdown",
        "enterRandEvent",
        "enterOperatorEvent",
        "enterFunctionEvent",
        "enterConstantEvent",
        "enterBracketEvent",
        "enterSpecialFuncEvent",
        "moveLeft",
        "moveRight",
        "copyResultToClipboard",
        "copyClipboard2Result",
        "allElection",
        "shear",
        "deleteText",
        "Undo",
        "addUndo",
        "Redo",
        "initTheme",
        "setResultFalse",
        "replaceSelection",
        "getInputEdit",
        "InputEdit*",
        "revisionResults",
        "QModelIndex",
        "index",
        "hisRevisionResults",
        "Quantity",
        "ans",
        "judgeinput",
        "handleTextChanged",
        "pointCheckAfterDelete",
        "onSeparateChange"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyPress'
        QtMocHelpers::SignalData<void(QKeyEvent *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'clearStateChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'themeChange'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'turnDeg'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fEStateChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'enterNumberEvent'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'enterSymbolEvent'
        QtMocHelpers::SlotData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'enterPointEvent'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterBackspaceEvent'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterClearEvent'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterEqualEvent'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterPercentEvent'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterDegEvent'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'enterFEEvent'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'enterRandEvent'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterOperatorEvent'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'enterFunctionEvent'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'enterConstantEvent'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'enterBracketEvent'
        QtMocHelpers::SlotData<void(const int &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'enterSpecialFuncEvent'
        QtMocHelpers::SlotData<void(const QString &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'moveLeft'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveRight'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyResultToClipboard'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyClipboard2Result'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'allElection'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'shear'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteText'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Undo'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addUndo'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Redo'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'initTheme'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'setResultFalse'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'replaceSelection'
        QtMocHelpers::SlotData<void(QString)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'getInputEdit'
        QtMocHelpers::SlotData<InputEdit *()>(40, 2, QMC::AccessPublic, 0x80000000 | 41),
        // Slot 'revisionResults'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 },
        }}),
        // Slot 'hisRevisionResults'
        QtMocHelpers::SlotData<void(const QModelIndex &, Quantity)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 43, 44 }, { 0x80000000 | 46, 47 },
        }}),
        // Slot 'judgeinput'
        QtMocHelpers::SlotData<bool()>(48, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'handleTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'pointCheckAfterDelete'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSeparateChange'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SciExpressionBar, qt_meta_tag_ZN16SciExpressionBarE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SciExpressionBar::staticMetaObject = { {
    QMetaObject::SuperData::link<DWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SciExpressionBarE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SciExpressionBarE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16SciExpressionBarE_t>.metaTypes,
    nullptr
} };

void SciExpressionBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SciExpressionBar *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyPress((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 1: _t->clearStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->themeChange((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->turnDeg(); break;
        case 4: _t->fEStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->enterNumberEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->enterSymbolEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->enterPointEvent(); break;
        case 8: _t->enterBackspaceEvent(); break;
        case 9: _t->enterClearEvent(); break;
        case 10: _t->enterEqualEvent(); break;
        case 11: _t->enterPercentEvent(); break;
        case 12: _t->enterDegEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->enterFEEvent((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->enterRandEvent(); break;
        case 15: _t->enterOperatorEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->enterFunctionEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: _t->enterConstantEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->enterBracketEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 19: _t->enterSpecialFuncEvent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->moveLeft(); break;
        case 21: _t->moveRight(); break;
        case 22: _t->copyResultToClipboard(); break;
        case 23: _t->copyClipboard2Result(); break;
        case 24: _t->allElection(); break;
        case 25: _t->shear(); break;
        case 26: _t->deleteText(); break;
        case 27: _t->Undo(); break;
        case 28: _t->addUndo(); break;
        case 29: _t->Redo(); break;
        case 30: _t->initTheme((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->setResultFalse(); break;
        case 32: _t->replaceSelection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 33: { InputEdit* _r = _t->getInputEdit();
            if (_a[0]) *reinterpret_cast<InputEdit**>(_a[0]) = std::move(_r); }  break;
        case 34: _t->revisionResults((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 35: _t->hisRevisionResults((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Quantity>>(_a[2]))); break;
        case 36: { bool _r = _t->judgeinput();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: _t->handleTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: _t->pointCheckAfterDelete(); break;
        case 39: _t->onSeparateChange(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SciExpressionBar::*)(QKeyEvent * )>(_a, &SciExpressionBar::keyPress, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SciExpressionBar::*)(bool )>(_a, &SciExpressionBar::clearStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SciExpressionBar::*)(int )>(_a, &SciExpressionBar::themeChange, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SciExpressionBar::*)()>(_a, &SciExpressionBar::turnDeg, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SciExpressionBar::*)(bool )>(_a, &SciExpressionBar::fEStateChanged, 4))
            return;
    }
}

const QMetaObject *SciExpressionBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SciExpressionBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16SciExpressionBarE_t>.strings))
        return static_cast<void*>(this);
    return DWidget::qt_metacast(_clname);
}

int SciExpressionBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 40;
    }
    return _id;
}

// SIGNAL 0
void SciExpressionBar::keyPress(QKeyEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void SciExpressionBar::clearStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SciExpressionBar::themeChange(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void SciExpressionBar::turnDeg()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void SciExpressionBar::fEStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
